﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.IO.Ports;
using System.Windows.Forms;

namespace DWBattMain
{
    // Program to Test Rechargeable Batteries v1.00, Feb 28, 2010
    // Copyright 2010, may be used without charge but cannot be sold

    public partial class frm_Main : Form
    {
        // debug mode 
        public double[] dDebugValues = new double[100];
        public bool bDebugMode = false;

        public const string strRelayOn = "$1DOFF";
        public const string strRelayOff = "$1DO00";
        public const string strReadAD1 = "$1RD";
        public const string strReadAD1Setup = "$1RS";

        public string crlf = ((char)10).ToString()  + ((char)13).ToString();
        public string nl = ((char)10).ToString() + ((char)13).ToString();
        public string cr = ((char)13).ToString();

        public bool bSendADCommand = false;                     // flag for auto mode A/D read
        public bool bRunningBatteryTest = false;                // flag 

        // essentially constants for calculations
        public double dChannelOneLoadResistance = 16.29029;
        public double dBatteryPackNominalVoltage = 18.000;
        public double dBatteryCellOneVolt = 15.000;             // 18/1.2
        public double dRelayShutOffVoltage = 14.5;
        public double dScaleFactorChan1 = 2.030321;
        public int iDataSaveInterval = 1;

        // calculations variables
        public double dInitialBattVoltage = 0;
        public int iSecondsElapsed = 0;
        public double dAverageCurrentToCellOne;          // from first batt volt to cell one volt = sum/iSecondsElapsed
        public int iTimeToCellOneVoltageDrop;            // time (in secs) from first batt volt to cell one volt
        public double dVoltageDropToCellOne;             // voltage drop from first batt volt to cell one volt
        public double dCalculatedMAHCapacity;            // calc from dTimeToCellOneVoltageDrop/dAverageCurrent
        public bool bGotFinalCalcValuesFlag;             // indicator that we tripped below cell one voltage
 
        // variables for measurement timer routine
        public double dtmr_AvgCurrent = 0;



         // ================================================================================================================
       public frm_Main()
        {
            InitializeComponent();
        }

       private void frm_Main_Load(object sender, EventArgs e)
       {
           // Read the ini file for initial values
           ReadBattINIFile();
           // initialize serial port
           InitializeSerialPort();
       }

       public void btn_StartAcquisition_Click(object sender, EventArgs e)
       {
           // stop reading A/D if was
           bSendADCommand = false;
           btn_ReadAD1.Text = "Read Voltage Chan 1";
           // see if test running or not
           if (bRunningBatteryTest == false) // test not started yet
           {
               bRunningBatteryTest = true;
               // disable the other buttons while test is running
               btn_ReadAD1.Enabled = false;
               btn_ReadSetup.Enabled = false;
               btn_ReadData1.Enabled = false;
               // now get the latest constants/settings (load resistance, scale factor, etc)
               ReadBattINIFile();
               // initialize the test
               btn_StartAcquisition.Text = "&Stop Battery Test";
               lbl_Status.Text = "Battery Test Running";
               WriteBatteryDataFileHeader();
               // debugging mode first 
               if (bDebugMode)
               {
                   tmr_SecondsCount.Interval = 200;
                   dDebugValues[0] = 18;
                   for (int i = 1; i < 100; i++)
                   {
                       dDebugValues[i] = dDebugValues[i - 1] - 0.05;
                   }
               }
               //'activate relay via Output D0
               serialPort1.Write("$1DOFF" + nl);
               btn_RelayOnOff1.Text = "Turn Off Relay";
               // enable the timer to get the incoming serial data and process it
               tmr_SecondsCount.Enabled = true;
           }
           else // stop the test
           { bRunningBatteryTest = false; }
       }
       private void tmr_SecondsCount_Tick(object sender, EventArgs e)
       {
           double dBatteryDataValue = 0;
           //If the BatteryTest is running, this timer tells when each second is up
           iSecondsElapsed++;
           //read A/D data from serial port by requesting A/D read
           serialPort1.Write("$1RD" + nl);
           WaitXSeconds(.05);
           if (bDebugMode)
                { dBatteryDataValue = dDebugValues[iSecondsElapsed -1];}
           else
                {dBatteryDataValue = double.Parse(strADValueScale(strReadADLineFromSerialPort("$1RD"), dScaleFactorChan1));}
           // only continue if we had a valid reading
           if (dBatteryDataValue > 0.1)
           {
               // if this is the first measurement then save this batt value for the calcs
               if (iSecondsElapsed == 1)
               { dInitialBattVoltage = dBatteryDataValue; }
               if (bRunningBatteryTest == true)
               {
                   // add this measurement to the list
                   lstbx_SerData.Items.Add("AD1 Volts = " + String.Format("{0:###.000}", dBatteryDataValue));
                   //lstbx_SerData.Items.Add("AD1 Volts = " + dBatteryDataValue.ToString());
                   lstbx_SerData.SelectedIndex = lstbx_SerData.Items.Count - 1;
                   // add up the current values 
                   dtmr_AvgCurrent = dtmr_AvgCurrent + (dBatteryDataValue / dChannelOneLoadResistance);
                   // update the displayed data
                   DisplayBatteryTestData(dBatteryDataValue);
                   // compute and update battery data values
                   WriteBatteryData(iSecondsElapsed, dBatteryDataValue, dChannelOneLoadResistance);
               }
               // see if we are at the Cell One Volt threshold
               if ((dBatteryDataValue <= dBatteryCellOneVolt) && (bGotFinalCalcValuesFlag == false))
               // freeze the final calc values
               {
                   dAverageCurrentToCellOne = dtmr_AvgCurrent / iSecondsElapsed;    // avg current to when cells = 1v
                   dVoltageDropToCellOne = dInitialBattVoltage - dBatteryDataValue; // voltage drop to cells=1v
                   iTimeToCellOneVoltageDrop = iSecondsElapsed;
                   bGotFinalCalcValuesFlag = true;
               }
               // if we are done then shutoff the relay and calc final data, write to file
               if ((dBatteryDataValue < dRelayShutOffVoltage) || (bRunningBatteryTest == false))
               {
                   WriteFinalCalculatedValuesToFile(iTimeToCellOneVoltageDrop, dInitialBattVoltage, dVoltageDropToCellOne, dAverageCurrentToCellOne);
                   // now reset everything
                   iSecondsElapsed = 0;
                   dtmr_AvgCurrent = 0;
                   bGotFinalCalcValuesFlag = false;
                   // enable btns again
                   btn_ReadAD1.Enabled = true;
                   btn_ReadSetup.Enabled = true;
                   btn_ReadData1.Enabled = true;
                   // turnoff relay
                   WaitXSeconds(.1);
                   serialPort1.Write("$1DO00" + nl);
                   btn_RelayOnOff1.Text = "Turn On Relay";
                   // set acq btn to not running
                   btn_StartAcquisition.Text = "&Start Battery Test";
                   // stop battery test run timer
                   tmr_SecondsCount.Enabled = false;
                   bRunningBatteryTest = false;
                   lbl_Status.Text = "Battery Data Written and Battery Test Stopped";
               }
           }
           // allow other stuff to run
           Application.DoEvents();
       }

        private void frm_Main_FormClosed(object sender, FormClosedEventArgs e)
        {
            //stop all actions and close the port
            bSendADCommand = false;
            bRunningBatteryTest = false;
            serialPort1.Close();
        }


        private void DisplayBatteryTestData(double dBatteryDataVal)
        {
            TimeSpan tsElapsedTime = new TimeSpan(0, 0, iSecondsElapsed);
            lbl_ElapsedTime.Text = tsElapsedTime.ToString();
            lbl_SecsCount.Text = iSecondsElapsed.ToString();
            lbl_Voltage.Text = String.Format("{0:###.000}", dBatteryDataVal);
            lbl_Current.Text = String.Format("{0:###.000}", dBatteryDataVal / dChannelOneLoadResistance);
            lbl_CellVoltage.Text = String.Format("{0:###.000}", dBatteryDataVal / (dBatteryPackNominalVoltage / 1.2));
        }

        private void UpdateBatteryTestDataValues()
            {   //here is what we update and later save
                //CTime, InitBV, NowBattV, TVDrop, PVDrop, ElTime, VDropPerMin, MinsPer1VDrop, AvgCurrent, MAH, TimeStep
            }


        private void btn_RelayOnOff1_Click(object sender, EventArgs e)
        {
            //Console.WriteLine("Relay Btn Pushed");
            if (btn_RelayOnOff1.Text.Equals( "Turn On Relay 1"))
            {//activate relay via Output D0
                serialPort1.Write("$1DOFF" + nl);
                btn_RelayOnOff1.Text = "Turn Off Relay 1";
                lbl_Status.Text = "Battery Load Relay On";

            }
            else
            {//deactivate relay via Output D0
                serialPort1.Write("$1DO00" + nl);
                btn_RelayOnOff1.Text = "Turn On Relay 1";
                lbl_Status.Text = "Battery Load Relay Off";
            }

        }

        private void btn_ReadAD1_Click(object sender, EventArgs e)
        {
            // read and display scaled a/d value, needs scale factor
            double dScaleFactorCh1 = 2.030321; //default
            dScaleFactorCh1 = double.Parse(GetStringFromINIFile("Setup", "Channel One Voltage Scale Factor", dScaleFactorCh1.ToString(), Application.StartupPath + "\\dwb.ini"));

            //read A/D data from module continuously via serial port
            //Console.WriteLine("Btn Read AD");
            btn_HiddenFocus.Focus();    // otherwise takes multiple btn_ReadAD1 hits
            if (btn_ReadAD1.Text.Equals("Read Voltage Chan 1")) //not reading so start
            {
                bSendADCommand = true;  // for start of loop to read a/d
                btn_ReadAD1.Text = "Stop Read Ch1";
                while (bSendADCommand == true)
                {
                    serialPort1.Write("$1RD" + nl);
                    lstbx_SerData.Items.Add("AD1 Voltage Read = " + strADValueScale(strReadADLineFromSerialPort("$1RD"), dScaleFactorCh1));
                    lstbx_SerData.SelectedIndex = lstbx_SerData.Items.Count - 1;
                    WaitXSeconds(.2);
                }
            }
            else
            {
                bSendADCommand = false;
                btn_ReadAD1.Text = "Read Voltage Chan 1";
            }
        }

        private void btn_ReadSetup_Click(object sender, EventArgs e)
        {
             //read A/D module setup value from serial port
            serialPort1.Write("$1RS" + nl);
            lstbx_SerData.Items.Add("AD1 Setup Read = " + strReadADLineFromSerialPort("$1RS"));
       }

        private void btn_ReadData1_Click(object sender, EventArgs e)
        {
            // read A/D but display the raw data
            serialPort1.Write("$1RD" + nl);
            lstbx_SerData.Items.Add("AD1 Raw Volt Read = " + strReadADLineFromSerialPort("$1RS"));
        }

        // ================================================================================================================
        public static void WaitXSeconds(double SecondsToWait)
        {
            DateTime startTime1 = DateTime.Now;
            /* Read the end time. */
            DateTime stopTime1 = DateTime.Now;
            /* Compute the duration of this first task. */
            TimeSpan ElapsedTime = stopTime1 - startTime1;

            //Console.WriteLine("Waiting");
            while (ElapsedTime.TotalMilliseconds < (SecondsToWait * 1000))
            {
                /* Read the end time. */
                stopTime1 = DateTime.Now;
                ElapsedTime = stopTime1 - startTime1;
                Application.DoEvents();
            }
            //Console.WriteLine("Not Waiting");
        }







    }
}
