﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using System.IO;
using System.Runtime.InteropServices;
using System.Text;

namespace DWBattMain
{
    public partial class frm_Main 
    {
        //private string strAppDriveLetter = null;

        [DllImport("kernel32.dll")]
        static extern uint GetPrivateProfileString(
           string lpAppName,
           string lpKeyName,
           string lpDefault,
           StringBuilder lpReturnedString,
           uint nSize,
           string lpFileName);
        //NC = GetPrivateProfileString("Section", "KeyName", "Default", sb, 255, "C:\test.ini")

        [DllImport("kernel32.dll")]
        static extern uint WritePrivateProfileString(
           string lpAppName,
           string lpSectionName,
           string lpSendString,
           string lpFileName);
        //NC = WritePrivateProfileString("Section", "KeyName", "StringToWrite","C:\test.ini")


        // ================================================================================================================
        public static string GetStringFromINIFile(string strSectionName, string strValueName, string strDefaultValue, string strINIFileName)
        {
            //NC = GetPrivateProfileString("Section", "KeyName", "Default", sb, 255, "C:\test.ini")
            // populate all fields with lists from ini files.
            StringBuilder srb = new StringBuilder(100);
            uint uintRes = 0;
            uintRes = GetPrivateProfileString(strSectionName, strValueName, strDefaultValue, srb, 90, strINIFileName);
            return srb.ToString();
        }

        public static bool GetTrueFalseFromINIFile(string strSectionName, string strValueName, string strDefaultValue, string strINIFileName)
        {
            //NC = GetPrivateProfileString("Section", "KeyName", "Default", sb, 255, "C:\test.ini")
            // populate all fields with lists from ini files.
            StringBuilder srb = new StringBuilder(100);
            uint uintRes = 0;
            uintRes = GetPrivateProfileString(strSectionName, strValueName, strDefaultValue, srb, 90, strINIFileName);
            // now see if the string reads True or else
            if (srb.ToString().ToUpper() == "TRUE")
            { return true; }
            else
            { return false; }
        }

        public static void SetTrueFalseToINIFile(string strSectionName, string strValueName, bool bValue, string strINIFileName)
        {
            // write True or False as string to INI file
            uint uintRes = 0;
            uintRes = WriteStringToINIFile(strSectionName, strValueName, bValue.ToString(), strINIFileName);
            // now see if the string reads True or else
        }

        public static void SetUpperCaseTrueFalseToINIFile(string strSectionName, string strValueName, bool bValue, string strINIFileName)
        {
            // write True or False as string to INI file
            uint uintRes = 0;
            string strUpperCaseTrueFalse = bValue.ToString();
            strUpperCaseTrueFalse = strUpperCaseTrueFalse.ToUpper();
            uintRes = WriteStringToINIFile(strSectionName, strValueName, strUpperCaseTrueFalse, strINIFileName);
            // now see if the string reads True or else
        }

        public static uint WriteStringToINIFile(string strSectionName, string strValueName, string strValue, string strINIFileName)
        {
            //NC = WritePrivateProfileString("Section", "KeyName", "StringToWrite","C:\test.ini")
            // Write to INI File
            uint uintRes = 0;
            uintRes = WritePrivateProfileString(strSectionName, strValueName, strValue, strINIFileName);
            return uintRes;
        }


        // ================================================================================================================
        public  void ReadBattINIFile()
        {
            if (System.IO.File.Exists((Application.StartupPath + "\\dwb.ini")))
            {
                dChannelOneLoadResistance = double.Parse(GetStringFromINIFile("Setup", "Load Resistance Ch1", dChannelOneLoadResistance.ToString(), Application.StartupPath + "\\dwb.ini"));
                dBatteryPackNominalVoltage = double.Parse(GetStringFromINIFile("Setup", "Battery Pack Nominal Voltage", dBatteryPackNominalVoltage.ToString(), Application.StartupPath + "\\dwb.ini"));
                dScaleFactorChan1 = double.Parse(GetStringFromINIFile("Setup", "Channel One Voltage Scale Factor", dScaleFactorChan1.ToString(), Application.StartupPath + "\\dwb.ini"));
                iDataSaveInterval = int.Parse(GetStringFromINIFile("Setup", "Data Save Interval", iDataSaveInterval.ToString(), Application.StartupPath + "\\dwb.ini"));
                dRelayShutOffVoltage = double.Parse(GetStringFromINIFile("Setup", "Relay Shutoff Voltage", dRelayShutOffVoltage.ToString(), Application.StartupPath + "\\dwb.ini"));
 
            }
        }

        // ================================================================================================================
        private void WriteBatteryDataFileHeader()
        {
            if (System.IO.Directory.Exists((Application.StartupPath + "\\data")) == false)
                { System.IO.Directory.CreateDirectory(Application.StartupPath + "\\data"); }

            File.WriteAllText(Application.StartupPath + "\\data\\" + txtbx_DataFileName.Text + ".csv", "TimeStep, BattV, Current " + cr);
        }


        private void WriteBatteryData(int iSecondsElapsed, double dLatestBattValue, double dChannelOneLoadResistance)
        {   // save the following data every second 
            // ElapsedTime, LatestBattVoltage, AvgCurrent 
            TimeSpan tsElapsedTime = new TimeSpan(0, 0, iSecondsElapsed);

            string TempString = tsElapsedTime.ToString();   //elapsed time
            TempString = TempString + "," + String.Format("{0:#.000}", dLatestBattValue); // latest batt value measurement
            TempString = TempString + "," + String.Format("{0:#.000}", dLatestBattValue / dChannelOneLoadResistance); // current
            File.AppendAllText(Application.StartupPath + "\\data\\" + txtbx_DataFileName.Text + ".csv", TempString + cr);
        }


        public void WriteFinalCalculatedValuesToFile(int iTimeToCellOneVoltageDrop, double dInitialBattV, double dVoltageDropToCellOne, double dAvgCurrentToCellOneV)
        {
            // calc all the final values
            // ElapsedTime, InitialBattVoltage, LatestBattVoltage, TotalVoltageDrop, AvgCurrent, CalcMAH
            // We compute MAH value by determining how long it takes for battery to reach 1 volt per cell. 
            // We assume a nominal cell voltage of 1.2, number of cells = nominal batt voltage / 1.2 and then 
            //  1 volt/cell voltage = number of cells * 1. 
            // However, for that actual battery test, we continue to a value slightly below 1 volt/cell so that
            //  we can see what the end curve is for the battery. 

            // we need to know 
            //  - elapsed time in secs (same as # of readings) - iSecondsElapsed
            //  - first batt voltage measurement - dInitialBattV
            //  - the last batt voltage value - dLatestBattValue
            //  - the average current up to the cell=1v value
            // we compute the following;
            //  - total voltage drop from first batt value to last batt value (at cell=1v)
            //  - MAH by average current/elapsedtime
            double dElapsedTime = Convert.ToDouble(iTimeToCellOneVoltageDrop);
            double MAH = (dAvgCurrentToCellOneV * ((dElapsedTime) / 3600)) * 1000; // avg current (in amps) * time (in hours) = amp hours * 1000 = MAH (millamp hours)

            // now write the values to the file
            string FCalcHeader = " ElapsedTime, InitBV, EndBV, TVDrop, AvgCurrent, MAH" + cr;
            File.AppendAllText(Application.StartupPath + "\\data\\" + txtbx_DataFileName.Text + ".csv", FCalcHeader);

            TimeSpan tsElapsedTime = new TimeSpan(0, 0, iTimeToCellOneVoltageDrop);
            string FCalcValues = tsElapsedTime.ToString() + ",";
            FCalcValues = FCalcValues + String.Format("{0:###.000}", dInitialBattV) + ",";
            FCalcValues = FCalcValues + String.Format("{0:###.000}", (dInitialBattV - dVoltageDropToCellOne)) + ",";
            FCalcValues = FCalcValues + String.Format("{0:###.000}", dVoltageDropToCellOne) + ",";
            FCalcValues = FCalcValues + String.Format("{0:###.000}", dAvgCurrentToCellOneV) + ",";
            FCalcValues = FCalcValues + String.Format("{0:###.000}", MAH) + cr;
            File.AppendAllText(Application.StartupPath + "\\data\\" + txtbx_DataFileName.Text + ".csv", FCalcValues);
        }


    }
}
